class AuthToken {
  AuthToken({
    required this.accessToken,
    required this.idToken,
    required this.refreshToken,
    required this.tokenType,
    required this.expiryDate,
    required this.scope,
  });

  final String accessToken;
  final String idToken;
  final String refreshToken;
  final String tokenType;
  final DateTime expiryDate;
  final String scope;

  factory AuthToken.fromJson(Map<String, dynamic> json) => AuthToken(
        accessToken: json["access_token"],
        idToken: json["id_token"],
        refreshToken: json["refresh_token"],
        tokenType: json["token_type"],
        expiryDate: DateTime.parse(json["expiry_date"]),
        scope: json["scope"],
      );

  Map<String, dynamic> toJson() => {
        "access_token": accessToken,
        "id_token": idToken,
        "refresh_token": refreshToken,
        "token_type": tokenType,
        "expiry_date": expiryDate.toIso8601String(),
        "scope": scope,
      };
}

extension AuthTokenExtensions on AuthToken {
  bool isExpired() {
    return expiryDate.isBefore(DateTime.now().add(const Duration(minutes: -1)));
  }
}
