import 'dart:async';

import 'package:jwt_decoder/jwt_decoder.dart';

import '../../models/auth_profile.dart';
import '../../models/auth_token.dart';
import '../../models/auth_user_model.dart';
import '../../storage.dart';

class AuthRepository {
  final StreamController<AuthUserModel?> _controller = StreamController<AuthUserModel?>.broadcast();

  AuthUserModel? _auth;
  AuthUserModel? get auth => _auth;
  String? _udid;
  String? get udid => _udid;
  late AuthStorage _authStorage;

  Stream<AuthUserModel?> get authStream => _controller.stream.asBroadcastStream();

  AuthRepository({required AuthStorage authStorage}) {
    setAuth(null);
    _authStorage = authStorage;
    authStream.asBroadcastStream().listen(
      (model) {
        _auth = model;
        if (model != null) {
          _authStorage.saveAuthModel(model.authToken);
        } else {
          _authStorage.clearAuth();
        }
      },
    );
    _authStorage.getAuthModel().then(
      (token) {
        if (token != null) {
          setAuth(token);
        }
      },
    );
    _authStorage.getPersistentUdid().then((value) {
      _udid = value;
    });
  }

  void setAuth(AuthToken? token) {
    if (token != null) {
      var accessTokenModel = JwtDecoder.decode(token.accessToken);
      var idModel = JwtDecoder.decode(token.idToken);
      var model = AuthUserModel(
        authToken: token,
        profile: AuthProfile.fromJson(idModel),
        hashedUserId: accessTokenModel["saimobileapps-ca:hashedUserId"] ?? "",
        accountType: accessTokenModel["saimobileapps:id:user_type"] ?? "FSBO",
      );
      _auth = model;
      _controller.sink.add(
        model,
      );
    } else {
      _controller.sink.add(null);
    }
  }

  void updateProfile({
    String? email,
    String? profilePicture,
    String? accountType,
    String? name,
  }) {
    var auth = _auth;
    if (auth != null) {
      var model = auth.copyWith(
        accountType: accountType,
        profile: auth.profile.copyWith(
          email: email,
          name: name,
          pictureUrl: profilePicture,
        ),
      );
      _auth = model;
      _controller.sink.add(
        model,
      );
    }
  }

  Future<String> getPersistentUdid() async {
    return await _authStorage.getPersistentUdid();
  }
}
