import 'dart:convert';

import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:uuid/uuid.dart';

import '../../models/auth_token.dart';

abstract class AuthStorage {
  Future<AuthToken?> getAuthModel();
  Future<void> saveAuthModel(AuthToken token);
  Future<void> clearAuth();
  Future<String?> getNotificationsToken();
  Future<void> saveNotificationsToken(String? token);
  Future<String> getPersistentUdid();
}

class AuthStorageImpl implements AuthStorage {
  final String _authKey = "auth_token";
  final String _pushTokenKey = "push_token";
  final String _udidKey = "udid_key";

  final FlutterSecureStorage secureStorage;
  AuthStorageImpl({required this.secureStorage});
  @override
  Future<AuthToken?> getAuthModel() async {
    var token = await secureStorage.read(key: _authKey);
    if (token != null) {
      return AuthToken.fromJson(jsonDecode(token));
    }
    return null;
  }

  @override
  Future<void> saveAuthModel(AuthToken token) async {
    await secureStorage.write(key: _authKey, value: jsonEncode(token));
  }

  @override
  Future<void> clearAuth() async {
    await secureStorage.delete(key: _authKey);
  }

  @override
  Future<String?> getNotificationsToken() async {
    return await secureStorage.read(key: _pushTokenKey);
  }

  @override
  Future<void> saveNotificationsToken(String? token) async {
    await secureStorage.write(key: _pushTokenKey, value: token);
  }

  @override
  Future<String> getPersistentUdid() async {
    var udid = await secureStorage.read(key: _udidKey);
    if (udid == null) {
      udid = const Uuid().v4();
      await secureStorage.write(key: _udidKey, value: udid);
    }
    return udid;
  }
}
