import 'package:flutter/material.dart';

/// Service to retrieve the user's current locale without direct access to a [BuildContext]
/// Prefer to retrieve locale info directly through [Localizations.localeOf] when a context is readily available.
class LocalizationService {
  static const _en = "en_CA";
  static const _fr = "fr_CA";
  static const _networkLanguageHeaderEn = "en-CA";
  static const _networkLanguageHeaderFr = "fr-CA";

  late final GlobalKey _contextKey;

  void init({required GlobalKey localizationContextKey}) => _contextKey = localizationContextKey;

  String getLocale() {
    var context = _contextKey.currentContext;
    if (context != null) {
      var locale = Localizations.localeOf(context).languageCode;
      if (locale.toLowerCase() == "fr") {
        return _fr;
      }
    }
    return _en;
  }

  String getLanguageForSaiMobileAppsHeader() {
    var context = _contextKey.currentContext;
    if (context != null) {
      var locale = Localizations.localeOf(context).languageCode;
      if (locale.toLowerCase() == "fr") {
        return _networkLanguageHeaderFr;
      }
    }
    return _networkLanguageHeaderEn;
  }
}
