import '../../logging.dart';

// coverage:ignore-start
abstract class Tracker {
  final ErrorTracker error;
  final AnalyticsTracker analytics;
  Tracker({
    required this.error,
    required this.analytics,
  });
}

abstract class ErrorTracker {
  Future<void> recordError(
    dynamic exception,
    StackTrace? stack, {
    String reason,
    bool fatal = false,
  });
}

abstract class AnalyticsTracker {
  Future<void> logAction({
    required String action,
    required String pageType,
    String? cta,
    Map<String, dynamic>? customLabels,
    Map<String, dynamic>? dataParameters,
    MetricsErrorSource? errorSource,
  });
  Future<void> logEvent({
    required String name,
    Map<String, Object?>? parameters,
  });

  Future<void> logViewPromotionEvent({
    String? creativeName,
    String? creativeSlot,
    List<AnalyticsEvent>? items,
    String? locationId,
    String? promotionId,
    String? promotionName,
  });

  Future<void> logSelectPromotionEvent({
    String? creativeName,
    String? creativeSlot,
    List<AnalyticsEvent>? items,
    String? locationId,
    String? promotionId,
    String? promotionName,
  });

  Future<void> logPageViewEvent({
    required String screenName,
    String? pageType,
    String? cta,
    Map<String, String>? customLabels,
    Map<String, dynamic>? dataParameters,
    MetricsErrorSource? errorSource,
  });
}
 // coverage:ignore-end