import 'package:map_launcher/map_launcher.dart';

import 'models/available_map_apps.dart';
import 'models/map_location_position.dart';
import 'models/map_location_type.dart';

abstract class MapService {
  Future<List<AvailableMapApp>> getInstalledMaps();
  Future<void> launchMap({
    required MapLocationType mapType,
    required LocationMapPosition mapPosition,
    required String title,
  });
}

class MapServiceImpl implements MapService {
  @override
  Future<void> launchMap({
    required MapLocationType mapType,
    required LocationMapPosition mapPosition,
    required String title,
  }) async {
    await MapLauncher.showMarker(
      mapType: mapType.mapType,
      coords: Coords(mapPosition.latitude, mapPosition.longitude),
      title: title,
    );
  }

  @override
  Future<List<AvailableMapApp>> getInstalledMaps() async {
    final availableApps = await MapLauncher.installedMaps;
    List<AvailableMapApp> installedApps = [];
    for (var app in availableApps) {
      final availableApp = _getAvailableApp(app.mapName, app.mapType);
      if (availableApp != null) {
        installedApps.add(availableApp);
      }
    }
    return installedApps;
  }

  AvailableMapApp? _getAvailableApp(String mapName, MapType type) {
    final mapType = type.mapType;
    if (mapType == null) return null;
    return AvailableMapApp(mapName: mapName, mapType: mapType);
  }
}
