import 'package:connectivity_plus/connectivity_plus.dart';
import 'package:flutter/services.dart';

abstract class ConnectivityInfo {
  Stream<ConnectivityResult> get onConnectivityChanged;
  Future<bool> isConnected();
}

class ConnectivityInfoImpl implements ConnectivityInfo {
  final Connectivity connectivity;

  ConnectivityInfoImpl({
    required this.connectivity,
  });

  @override
  Stream<ConnectivityResult> get onConnectivityChanged =>
      connectivity.onConnectivityChanged;

  @override
  Future<bool> isConnected() async {
    try {
      var connection = await connectivity.checkConnectivity();
      return [ConnectivityResult.wifi, ConnectivityResult.mobile].contains(
        connection,
      );
    } on PlatformException catch (_) {
      rethrow;
    }
  }
}
