import 'dart:io';

import 'package:dio/dio.dart';
import 'package:dio_cache_interceptor/dio_cache_interceptor.dart';
import 'package:uuid/uuid.dart';

import '../../config/cache_config.dart';

class CacheInterceptor extends DioCacheInterceptor {
  final CacheConfig cacheConfig;
  final CacheOptions cacheOptions;
  CacheInterceptor({required this.cacheConfig, required this.cacheOptions}) : super(options: cacheOptions);
  @override
  void onRequest(
    RequestOptions options,
    RequestInterceptorHandler handler,
  ) async {
    if (cacheConfig.isCacheable(uri: options.uri)) {
      options.extra.addAll(
        cacheOptions
            .copyWith(
              policy: CachePolicy.request,
            )
            .toExtra(),
      );

      super.onRequest(options, handler);
    } else {
      handler.next(options);
    }
  }

  @override
  void onResponse(Response response, ResponseInterceptorHandler handler) {
    if (cacheConfig.isCacheable(uri: response.requestOptions.uri)) {
      response.headers.add(HttpHeaders.etagHeader, const Uuid().v4());
      response.headers.add(
        HttpHeaders.cacheControlHeader,
        CacheControl(
          maxAge: cacheConfig.maxAgeForRequest(uri: response.requestOptions.uri),
        ).toHeader(),
      );
      super.onResponse(response, handler);
    } else {
      handler.next(response);
    }
  }
}
