// coverage:ignore-file
import 'package:dio/dio.dart';

/// For Integration testing only
class IntegrationInterceptor extends InterceptorsWrapper {
  Map<String, ResponseOverride>? responseOverrides;
  IntegrationInterceptor({this.responseOverrides});

  @override
  void onRequest(RequestOptions options, RequestInterceptorHandler handler) async {
    var path = options.path;
    if (responseOverrides != null) {
      for (var entry in responseOverrides!.entries) {
        if (path.contains(RegExp(entry.key))) {
          var responseOverride = entry.value;
          return handler.resolve(
            Response(
              data: await responseOverride.dataResolver(),
              statusCode: responseOverride.statusCode,
              requestOptions: options,
            ),
          );
        }
      }
    }
    super.onRequest(options, handler);
  }
}

class ResponseOverride {
  final String path;
  final int statusCode;
  final Future<dynamic> Function() dataResolver;

  ResponseOverride({
    required this.path,
    required this.statusCode,
    required this.dataResolver,
  });
}
