import 'dart:io';

import 'package:device_info_plus/device_info_plus.dart';
import 'package:dio/dio.dart';
import 'package:flutter/foundation.dart';

import '../../../config.dart';
import '../../../localization.dart';
import '../../../storage.dart';
import '../../platform/platform_service.dart';

class SaiMobileAppsInterceptor extends InterceptorsWrapper {
  final LocalizationService localizationService;
  final AuthStorage authStorage;
  String? userAgent;
  String? persistentUdid;
  SaiMobileAppsInterceptor({
    required this.localizationService,
    required this.authStorage,
  });

  @override
  void onRequest(RequestOptions options, RequestInterceptorHandler handler) async {
    if (userAgent == null) {
      await _initUserAgent();
    }
    persistentUdid ??= await authStorage.getPersistentUdid();

    // options.headers.putIfAbsent(CustomHttpHeaders.udidHeader, () => persistentUdid);
    // options.headers.putIfAbsent(CustomHttpHeaders.traceNumberHeader, () => const Uuid().v4());
    options.headers.putIfAbsent(
      HttpHeaders.acceptLanguageHeader,
      () => localizationService.getLanguageForSaiMobileAppsHeader(),
    );
    options.headers.putIfAbsent(HttpHeaders.userAgentHeader, () => userAgent);
    super.onRequest(options, handler);
  }

  Future<void> _initUserAgent() async {
    const String unknownSystemVersion = "Unknown System Version";
    const String unknownModel = "Unknown Model";
    PlatformService platformService = Injector.get();
    DeviceInfoPlugin deviceInfo = DeviceInfoPlugin();
    var locale = Platform.localeName;
    late String appName;
    String version = platformService.appVersion;

    String? systemName;
    String? systemVersion;
    String? model;
    var platform = defaultTargetPlatform;
    if (platform == TargetPlatform.iOS) {
      const String unknownSystemName = "Unknown System Name";
      IosDeviceInfo iosInfo = await deviceInfo.iosInfo;

      appName = platformService.appName;
      systemName = iosInfo.systemName ?? unknownSystemName;
      systemVersion = iosInfo.systemVersion ?? unknownSystemVersion;
      model = iosInfo.model ?? unknownModel;
    } else if (platform == TargetPlatform.android) {
      const String unknownManufacturer = "Unknown Manufacturer";
      AndroidDeviceInfo androidInfo = await deviceInfo.androidInfo;

      appName = platformService.packageName;
      systemName = "Android";
      systemVersion = androidInfo.version.release ?? unknownSystemVersion;
      var manufacturer = androidInfo.manufacturer ?? unknownManufacturer;
      var androidModel = androidInfo.model ?? unknownModel;
      model = "$manufacturer $androidModel";
    }
    userAgent = "$appName $version ($model; $systemName $systemVersion; $locale)";
  }
}
