import 'package:permission_handler/permission_handler.dart';

import 'permission_result.dart';
import 'permission_type.dart';

abstract class PermissionService {
  Future<PermissionResult> checkPermission(PermissionType permissionType);
  Future<PermissionResult> requestPermission(PermissionType permissionType);
  void openSettings();
}

class PermissionServiceImpl extends PermissionService {
  @override
  Future<PermissionResult> checkPermission(PermissionType permissionType) async {
    final permission = permissionType.type;
    final status = await permission.status;
    return _getPermissionStatus(status);
  }

  @override
  Future<PermissionResult> requestPermission(PermissionType permissionType) async {
    final permission = permissionType.type;
    var status = await permission.status;
    if (status != PermissionStatus.granted) {
      status = await permission.request();
    }
    return _getPermissionStatus(status);
  }

  PermissionResult _getPermissionStatus(PermissionStatus status) {
    switch (status) {
      case PermissionStatus.granted:
        return PermissionResult.granted;
      case PermissionStatus.denied:
        return PermissionResult.denied;
      case PermissionStatus.permanentlyDenied:
      case PermissionStatus.restricted:
        return PermissionResult.permanentlyDenied;
      case PermissionStatus.limited:
        return PermissionResult.limited;
      default:
        return PermissionResult.denied;
    }
  }

  @override
  void openSettings() {
    openAppSettings();
  }
}
