import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import 'swatches.dart';
import 'theme_components/circle_thumb_shape.dart';

export '../ui/theme_components/circle_thumb_shape.dart';

class SaiMobileAppsTheme {
  static ThemeData themeFor({required Brightness brightness}) {
    return brightness == Brightness.light ? _lightTheme : _darkTheme;
  }

  static ThemeData get _lightTheme {
    return ThemeData(
      fontFamily: "Larsseit",
      colorScheme: ColorScheme.fromSwatch(
        primarySwatch: Swatches.purple,
        accentColor: Swatches.red,
        backgroundColor: Colors.white,
        cardColor: Colors.white,
        errorColor: Swatches.red.shade600,
      ),
      primaryColor: Swatches.purple,
      elevatedButtonTheme: ElevatedButtonThemeData(
        style: _baseElevatedButtonStyle,
      ),
      textButtonTheme: TextButtonThemeData(
        style: _baseTextButtonStyle.copyWith(
          foregroundColor: MaterialStateProperty.resolveWith(
            (states) {
              if (states.contains(MaterialState.disabled)) {
                return Swatches.grey.shade200;
              }
              return Swatches.green.shade600;
            },
          ),
        ),
      ),
      tabBarTheme: _baseTabBarTheme.copyWith(
        labelColor: Swatches.grey,
        unselectedLabelColor: Colors.grey.shade400,
      ),
      textTheme: TextTheme(
        bodyText1: _baseBodyText1.copyWith(color: Swatches.grey.shade400),
        bodyText2: _baseBodyText2.copyWith(color: Swatches.grey),
        subtitle1: _baseSubtitle1.copyWith(color: Swatches.grey),
        subtitle2: _baseSubtitle2.copyWith(color: Swatches.grey),
        headline5: _baseHeadline5,
      ),
      checkboxTheme: _baseCheckboxTheme.copyWith(
        side: MaterialStateBorderSide.resolveWith(
          (states) {
            Color color = Swatches.grey.shade200;

            if (states.contains(MaterialState.selected) && !states.contains(MaterialState.disabled)) {
              color = Swatches.purple;
            }
            return BorderSide(color: color);
          },
        ),
        fillColor: MaterialStateProperty.resolveWith(
          (states) {
            if (states.contains(MaterialState.disabled)) {
              return Swatches.grey.shade200;
            }
            return Swatches.purple;
          },
        ),
      ),
      chipTheme: ChipThemeData(
        brightness: Brightness.light,
        padding: const EdgeInsets.all(8),
        disabledColor: Swatches.grey.shade200,
        selectedColor: Swatches.purple.shade50,
        secondarySelectedColor: Swatches.purple.shade50,
        checkmarkColor: Swatches.grey,
        backgroundColor: Colors.white,
        side: BorderSide(color: Swatches.grey.shade200),
        labelStyle: _baseChipLabelTextStyle.copyWith(color: Swatches.grey),
        secondaryLabelStyle: _baseChipLabelTextStyle.copyWith(
          fontWeight: FontWeight.bold,
          color: Swatches.purple.shade400,
        ),
      ),
      inputDecorationTheme: _baseInputDecorationTheme.copyWith(
        focusedBorder: OutlineInputBorder(
          borderSide: BorderSide(
            color: Swatches.blue,
          ),
        ),
        disabledBorder: OutlineInputBorder(
          borderSide: BorderSide(
            color: Swatches.grey.shade200,
          ),
        ),
        fillColor: Swatches.grey.shade100,
        hintStyle: TextStyle(
          color: Swatches.grey.shade400,
        ),
        errorBorder: OutlineInputBorder(
          borderSide: BorderSide(
            color: Swatches.red.shade600,
          ),
        ),
        errorStyle: _baseBodyText2.copyWith(
          color: Swatches.red.shade600,
          fontWeight: FontWeight.bold,
        ),
      ),
      snackBarTheme: _baseSnackBarTheme.copyWith(
        contentTextStyle: const TextStyle(color: Colors.white),
        actionTextColor: Colors.white,
        backgroundColor: Swatches.grey,
      ),
      bottomSheetTheme: _baseBottomSheetTheme.copyWith(
        backgroundColor: Colors.white,
      ),
      appBarTheme: _baseAppBarTheme.copyWith(
        systemOverlayStyle: SystemUiOverlayStyle.dark,
        backgroundColor: Colors.white,
        foregroundColor: Swatches.grey,
      ),
      primaryColorDark: Swatches.purple,
      indicatorColor: Swatches.purple,
      errorColor: Swatches.red.shade600,
      iconTheme: IconThemeData(color: Swatches.purple),
      shadowColor: Swatches.grey.withOpacity(0.1),
      dividerColor: Swatches.grey.shade200,
      scaffoldBackgroundColor: Colors.white,
      toggleableActiveColor: Swatches.purple,
      dialogTheme: const DialogTheme(
        backgroundColor: Colors.white,
      ),
      pageTransitionsTheme: _basePageTransitionTheme,
      radioTheme: RadioThemeData(
        fillColor: MaterialStateProperty.resolveWith(
          (states) {
            if (states.contains(MaterialState.disabled)) {
              return Swatches.grey.shade200;
            }
            return Swatches.purple;
          },
        ),
      ),
      sliderTheme: _baseSliderTheme.copyWith(
        thumbColor: Swatches.purple,
        inactiveTrackColor: Swatches.grey.shade200,
      ),
      switchTheme: SwitchThemeData(
        trackColor: MaterialStateProperty.resolveWith(
          (states) {
            if (states.contains(MaterialState.disabled)) {
              return Swatches.grey.shade100;
            }
            if (states.contains(MaterialState.selected)) {
              return Swatches.purple.shade100;
            }
            return Swatches.grey.shade200;
          },
        ),
        thumbColor: MaterialStateProperty.resolveWith(
          (states) {
            if (states.containsAll([MaterialState.disabled, MaterialState.selected])) {
              return Swatches.grey.shade200;
            }
            if (states.contains(MaterialState.selected)) {
              return Swatches.purple.shade400;
            }
            return Colors.white;
          },
        ),
      ),
    );
  }

  static ThemeData get _darkTheme {
    return ThemeData.dark();
  }

  static const TextStyle _baseTextStyle = TextStyle(fontFamily: "Larsseit");

  static final TextStyle _baseBodyText1 = _baseTextStyle.copyWith(fontSize: 16, height: 1.5, leadingDistribution: TextLeadingDistribution.even);
  static final TextStyle _baseBodyText2 = _baseTextStyle.copyWith(fontSize: 14, height: 1.5, leadingDistribution: TextLeadingDistribution.even);
  static final TextStyle _baseSubtitle1 = _baseTextStyle.copyWith(fontSize: 16);
  static final TextStyle _baseSubtitle2 = _baseTextStyle.copyWith(fontWeight: FontWeight.w500, fontSize: 18);
  static final TextStyle _baseHeadline5 = _baseTextStyle.copyWith(fontWeight: FontWeight.w500, fontSize: 24);
  static final TextStyle _baseChipLabelTextStyle = _baseBodyText1;

  static final AppBarTheme _baseAppBarTheme = AppBarTheme(
    elevation: 0.0,
    titleTextStyle: _baseHeadline5.copyWith(
      fontSize: 18,
      color: Swatches.grey,
    ),
  );
  static final CheckboxThemeData _baseCheckboxTheme = CheckboxThemeData(
    shape: RoundedRectangleBorder(
      borderRadius: BorderRadius.circular(4),
    ),
  );

  static final ButtonStyle _baseElevatedButtonStyle = ButtonStyle(
    textStyle: MaterialStateProperty.all(_baseBodyText1.copyWith(fontWeight: FontWeight.w500)),
    minimumSize: MaterialStateProperty.all(
      const Size(120, 48),
    ),
  );

  static final ButtonStyle _baseTextButtonStyle = ButtonStyle(
    textStyle: MaterialStateProperty.all(_baseBodyText1),
    minimumSize: MaterialStateProperty.all(
      const Size(0, 48),
    ),
  );

  static final TabBarTheme _baseTabBarTheme = TabBarTheme(
    labelPadding: EdgeInsets.zero,
    labelStyle: _baseBodyText1,
    unselectedLabelStyle: _baseTextStyle,
  );

  static const InputDecorationTheme _baseInputDecorationTheme = InputDecorationTheme(
    border: OutlineInputBorder(),
  );

  static const SnackBarThemeData _baseSnackBarTheme = SnackBarThemeData(
    behavior: SnackBarBehavior.floating,
  );

  static const BottomSheetThemeData _baseBottomSheetTheme = BottomSheetThemeData(
    shape: RoundedRectangleBorder(
      borderRadius: BorderRadius.only(
        topLeft: Radius.circular(10),
        topRight: Radius.circular(10),
      ),
    ),
  );

  static const SliderThemeData _baseSliderTheme = SliderThemeData(
    trackHeight: 6,
    overlayShape: RoundSliderOverlayShape(overlayRadius: 1),
    thumbShape: CircleThumbShape(thumbRadius: 15),
  );

  static const PageTransitionsTheme _basePageTransitionTheme = PageTransitionsTheme(
    builders: {
      TargetPlatform.iOS: CupertinoPageTransitionsBuilder(),
      TargetPlatform.android: CupertinoPageTransitionsBuilder(),
    },
  );
}
